# Supports Organizer.rb
=begin rdoc
= Loose Geometry to Components
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.smustard.com
Name:: Loose Geometry to Components
Version:: v1.2
SU Version:: Developed on SU 7, should be compatible with 6 and 5 at least.
Date:: Augsut 28, 2009
Description:: This script Components all loose geometry into multiple Components.
Usage:: Run the script from Plugins > Chris Fullmer Tools > Loose Geometry to Components.  If the user has geometry selected, it will work the selection.  If nothing is slected, it will work with all loose geometry.  It will not group components or existing Components.  It makes a new group for all connected geometry.  So if there are 20 separate boxes, it will make 20 separate Components - 1 for each box.  I use this to quickly group large amounts of quickly made 3d buildings over bery large areas of land.  This way I do not have so much loose geometry floating around.
History::
0.1:: 2009-02-07
   * First release
1.0:: 2009-02-13
   * Update rdoc compatiblity
   * New license info to conform to Smustards EULA
1.1:: 2009-03-09
   * Organizer.rb support. If you place this script into an organizer directory, it will appear in the correct submenu.  If its just in the plugins folder, it will appear in "Chris Fullmer Tools" submenu of the Plugins Menu."
1.2:: 2009-08-28
   * Fixed a major bug where the script was not working with selections.
   * Fixed some internal coding that might make it slightly faster.
   * Added an entity counter to the Status Bar so you can see the script working.
   * Changed the Undo title from "Make Components" to "Loose to Components" (this is the title that you see in the Edit > Undo command).
=end

require 'sketchup.rb'

module Clf_looseComponents

	# Make Components of all geometry in the selection set or active_entities.
	def Clf_looseComponents.main
		model = Sketchup.active_model
		@entities = model.active_entities
		sel = model.selection
		if sel[0]
			ents = sel.to_a
		else
			ents = @entities.to_a
		end
		
		if Sketchup.version_number < 7000000
			model.start_operation "Loose to Components"
		else
			model.start_operation "Loose to Components", true
		end
		
		#this divides the given entities into Components of 1000 and then sends them to be turned into Components
		chunk = 1000
		ents_count = ents.length
		count = (ents.length/chunk)+1
		new_arr = []
		@top_level = ents[0].parent
		count.times do |e|
			place_number = (e*chunk)
			new_arr = ents[place_number,chunk]
			Clf_looseComponents.maker(new_arr, place_number, ents_count)
		end
		model.commit_operation
	end
	
	#this takes each given entity, finds all_connected, then adds all that to a Components, unless the given entity is already a part of a group
	#That would happen when an entity it was connected to was already sent to this method in a previous block.
	def Clf_looseComponents.maker(ents, place_number, ents_count)
		total_ents = ents.length
		while ents.length > 0
			Sketchup.set_status_text "Processing #{(total_ents-ents.length)+place_number} of #{ents_count} entities.", SB_PROMPT
			if ((ents[0].is_a? Sketchup::Edge) || (ents[0].is_a? Sketchup::Face)) && ents[0].parent == @top_level
				to_group = ents[0].all_connected
				ents = ents - to_group
				group = @entities.add_group(to_group)
				group.to_component ##### comente essa linha se deseja que ao inves de componentes, o scrip gere grupos
			else
				ents = ents - [ents[0]]
			end
		end
	end # maker
end


#----------Menu system----------
if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		organizer.add_item("Loose Geometry to Components") { Clf_looseComponents.main  }
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
		$clf_tools_menu.add_item("Loose Geometry to Components") { Clf_looseComponents.main  }
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)